# GNU Enterprise Application Server - List Object
#
# Copyright 2001 Free Software Foundation
#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it 
# and/or modify it under the terms of the GNU General Public 
# License as published by the Free Software Foundation; either 
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be 
# useful, but WITHOUT ANY WARRANTY; without even the implied 
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public 
# License along with program; see the file COPYING. If not, 
# write to the Free Software Foundation, Inc., 59 Temple Place 
# - Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: geasList.py,v 1.11 2002/09/18 14:07:45 siesel Exp $

from gnue.common import GDataSource,GConditions
import geasInstance
import string

# =============================================================================
# List class
# =============================================================================

class geasList:

  # ---------------------------------------------------------------------------
  # Initalize
  # ---------------------------------------------------------------------------

  def __init__ (self, session, classname):
    self._session = session
    self._classname = classname
    self._prefetch = []
    self._conditions = []

  # ---------------------------------------------------------------------------
  # Actually these 3 only set variables
  # ---------------------------------------------------------------------------

  def setPrefetch (self, prefetch):
    self._prefetch = prefetch
    

  # conditions = an list consisting of condition elements written down in
  # prefix  notation i.e. (a=12) AND (c=121) -> AND = a 12 = c 121
  # every element of the list is a tuple containing the following parts:
  # type (eq,and,const...)  value  i.e. ('eq','') or ('const',12)

  def setConditions (self, conditions):    
    self._conditions = conditions
    
    # now building GCondition tree
    self._conditionTree = GConditions.buildTreeFromPrefix(conditions)

    
      
  def setSort (self, sort):
    self._sort = sort

  # ---------------------------------------------------------------------------
  # Populate the list with data from the database backend
  # ---------------------------------------------------------------------------

  def populate (self):
    self._datasource = GDataSource.GDataSource ()
    # TODO: get the table name from the class name
    self._datasource.buildObject (name = "",
    				  database = self._session._database,
    				  table = self._classname)
    self._datasource.setConnectionManager (self._session._connections)

    # bug fix should be moved into common after the freeze
    self._datasource._datasourceDictionary={}

    # set the field to sort (has to be called before phase init
    if hasattr(self,"_sort"):
      self._datasource.order_by=string.joinfields(self._sort,",")
    
    self._datasource.phaseInit ()
    # TODO: get the column names from the prefetch field names
    for field in self._prefetch:
      self._datasource.referenceField (field)

    if hasattr(self,"_conditionTree"):
      self._resultset = self._datasource.createResultSet(self._conditionTree)
    else:
      self._resultset = self._datasource.createResultSet()
      

  # ---------------------------------------------------------------------------
  # Get the first instance in the list
  # ---------------------------------------------------------------------------

  def firstInstance (self):
    if self._classname!='appserver_pytrigger':
      trigger=self._session._triggerMg.getTriggerByEvent \
               ('%s:pre_first_inst' % self._classname)
      if trigger!=None:
        trigger()
        
    if self._resultset.firstRecord () != 0:
      return geasInstance.geasInstance (self,  self._resultset.current)
    else:
      return None

  # ---------------------------------------------------------------------------
  # Get the next instance in the list
  # ---------------------------------------------------------------------------

  def nextInstance (self):
    if self._resultset.nextRecord () != 0:
      return geasInstance.geasInstance (self, self._resultset.current)
    else:
      return None

  # ---------------------------------------------------------------------------
  # insert a new empty instance in the list and return it 
  # ---------------------------------------------------------------------------

  def insertNewInstance (self):
    self._resultset.insertRecord ()
    return geasInstance.geasInstance (self, self._resultset.current)

  def getLength(self):
    pass

  def isEmpty(self):
    pass

  def isFirst(self):
    pass

  def isLast(self):
    pass

  def previousItem(self):
    pass
