# GNU Enterprise Application Server - RPC Server Application
#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2001-2002 Free Software Foundation
#
# $Id: geasRpcServer.py,v 1.13 2002/09/17 15:27:43 jcater Exp $

from geasList import *
from geasSession import *
import geasAuthentification

class geasSessionManager:
  def __init__(self):
    self._sessNo=0
    # use DB for authetification
    self._authAdapter=geasAuthentification.geasDBAuthAgent()

  def getNewSession(self):
    self._sessNo=self._sessNo+1
    sess=geasSession (self._connections)
    sess._database=self._database
    sess._authAdapter=self._authAdapter
    return sess

  def setConnections(self,connections):
    self._connections=connections

  def setDatabase(self,database):
    self._database=database

  def Shutdown(self):
    pass

  def Restart(self):
    pass

  def Status(self):
    if self._sessNo==0:
      return "Running (no sessions created yet)"
    else:
      return "Running (%s session created)" % self._sessNo

from gnue.common import GComm, GLoginHandler, GConfig
from gnue.common.FileUtils import openResource
from gnue.common.GServerApp import GServerApp
import time,os,sys
from gnue.appserver import VERSION

# =============================================================================
# RPC application class
# =============================================================================

class geasRpcServer(GServerApp):

  NAME = "GNUe Application Server"
  VERSION = VERSION
  USAGE = GServerApp.USAGE
  COMMAND = 'gnue-appserver'
  SUMMARY = "GNUe Application Server is a data and business rules server. " \
          + "It provides a backend to forms and reports. In an n-tier " \
          + "environment, GEAS holds all business rules/logic and provides " \
          + "data access to various database backends."

  COMMAND_OPTIONS = [
    [ 'rpctype',None,'rpc-type',1,"xmlrpc", 'type',
      _('Set the GNURPC connection type. The currently supported values ')+
      _("for <type> are 'xmlrpc' and 'pw_xmlrpc'. For more information on ")+
      _('GNURPC connection types have a look at ')+
      _('common/doc/RPC-abstraction. ')],
    [ 'database',None,'database',1,"gnue", 'name',
      _('Set the Database to use for loading and storing data to ')+
      _("<name>. Appserver will search for it in ")+
      _('connections.conf. ')],
    [ 'username',"u",'username',1,None, 'user',
      _('Set the username for the used database.')],
    [ 'password',"p",'password',1,None, 'password',
      _('Set the password for the used database.')]]


  ## set a list of transports
  def setTransports(self,transports):

    self._transports=transports


  def phaseInit(self):
    if self.OPTIONS["rpctype"]=="xmlrpc":

      print "Exporting our services via xmlrpc (port 8765) ..."
      self.setTransports({'xmlrpc':{ 'port': 8765 }})

    elif self.OPTIONS["rpctype"]=="pw_xmlrpc":

      print "Exporting our services via pw_xmlrpc (port 8765) ..."
      self.setTransports({'pw_xmlrpc':{ 'port': 8765 }})

    elif self.OPTIONS["rpctype"]=="sockets":

      # Sockets not working yet
      print "Exporting our services via sockets (port 8765) ..."
      self.setTransports({'sockets':{ 'port': 8765 }})

    else:

      # not transport protocol set. exiting
      print "The protocol you've set is currently not supported."
      sys.exit(-1)


  def run(self):
           
    # Create the various servers
    servers=GComm.bind (GConfig.getInstalledBase('install_prefix')+\
                        '/shared/grpc/GEAS.grpc',
                         self._transports,                         
                         {'SessionManager': self.requestSessionManager })

    # Daemonize (if appropriate)
    GServerApp.run(self)

    # Start the server for the different protocolls
    for key in servers.keys():
      servers[key].serveAsNewThread()      

    # wait for the servers shut down
    try:
      while 1==1:
        time.sleep(1999999)
    except KeyboardInterrupt:
      print "Appserver is shuting down....ok"
    

  def requestSessionManager(self):
    #print "SessionManager has started up."
    sm=geasSessionManager()
    loginhandler=testLoginHandler ()
    if self.OPTIONS["username"]:
      loginhandler.setLoginData(self.OPTIONS["username"],
                                self.OPTIONS["password"])
      
    self.connections.setLoginHandler (loginhandler)
    sm.setConnections(self.connections)
    sm.setDatabase(self.OPTIONS["database"])
    return sm

# =============================================================================
# Login Handler
# =============================================================================

class testLoginHandler (GLoginHandler.LoginHandler):
  def setLoginData(self,username,password):
    self._username=username
    self._password=password    
    
  def getLogin (self, *arguments):
    if hasattr(self,"_username"):
      return {"_username": self._username,
              "_password": self._password}
    
    # try to log into the database as the current user
    # works only on UNIX and with postgresql
    if sys.platform != 'win32':
      username = os.environ ["LOGNAME"]
    else:
      # use getpass module function getUser instead
      username = "gnue"
      
    return {"_username": username, "_password": "(none)"}


#
# main
#
if __name__ == '__main__':

  geas1 = geasRpcServer();

  geas1.phaseInit()

  print "\n... GNUe Application Server up and running ...\n"
  geas1.run();
  
