# GNU Enterprise Application Server - Session Object
#
# Copyright 2001 Free Software Foundation
#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it 
# and/or modify it under the terms of the GNU General Public 
# License as published by the Free Software Foundation; either 
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be 
# useful, but WITHOUT ANY WARRANTY; without even the implied 
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public 
# License along with program; see the file COPYING. If not, 
# write to the Free Software Foundation, Inc., 59 Temple Place 
# - Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: geasSession.py,v 1.10 2002/06/21 00:34:33 siesel Exp $

import geasList
import geasTrigger 
import geasAuthentification

# =============================================================================
# Session class
# =============================================================================

class geasSession:

  # ---------------------------------------------------------------------------
  # Initalize
  # ---------------------------------------------------------------------------

  def __init__ (self, connections):
    self.loggedIn = 0
    self._user = ""
    self._connections = connections
    self._database = "gnue"
    self._activelists = []
    self._authAdapter = geasAuthentification.geasAuthAgent()
    self._triggerMg = geasTrigger.geasPythonDBTriggerMg(self)
  # ---------------------------------------------------------------------------
  # Log into the application server
  # ---------------------------------------------------------------------------

  def login (self, user, password):
    # This username/password is for the Application Server, not for the
    # database.    
    self._user = user
    self.loggedIn = self._authAdapter.authentificate(self, user,
                                                     {'password':password} )
    return self.loggedIn

  # ---------------------------------------------------------------------------
  # Logout from the application server
  # ---------------------------------------------------------------------------

  def logout (self):
    # should the authAdapter be contacted?
    self.loggedIn = 0

  # ---------------------------------------------------------------------------
  # Create a new list of business objects of a given class
  # ---------------------------------------------------------------------------

  def createList (self, classname):
    
    # check if user has access rights for this list
    if self._authAdapter.hasAccess (self, self._user, classname):
      
      tr=self._triggerMg.getTriggerByEvent('%s:pre_new_list' % classname)
      if tr!=None:
        tr()
      # create new List
      newlist=geasList.geasList (self, classname)
      # every new list will be added to the _activelists list
      # which will be parsed for commit and rollback actions
      self._activelists.append(newlist)
      return newlist;
    
    else: # no access
      return None

  # ---------------------------------------------------------------------------
  # Commit the active transaction
  # ---------------------------------------------------------------------------

  def commit (self):
    for l in self._activelists:
      l._resultset.post()
      l._datasource.commit()

  # ---------------------------------------------------------------------------
  # Rollback the active transaction
  # ---------------------------------------------------------------------------

  def rollback (self):  
    for l in self._activelists:
      if hasattr(l,"_datasource"):
        l._datasource.rollback()    
