# GNU Enterprise Application Server - Test Program
#
# Copyright 2001 Free Software Foundation
#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it 
# and/or modify it under the terms of the GNU General Public 
# License as published by the Free Software Foundation; either 
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be 
# useful, but WITHOUT ANY WARRANTY; without even the implied 
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public 
# License along with program; see the file COPYING. If not, 
# write to the Free Software Foundation, Inc., 59 Temple Place 
# - Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: test.py,v 1.11 2002/09/18 14:07:45 siesel Exp $

import os
from gnue.common import GClientApp
from gnue.common import GLoginHandler
import geasSession

# =============================================================================
# Test application
# =============================================================================

class testApp (GClientApp.GClientApp):

  # ---------------------------------------------------------------------------
  # Create a session object
  # ---------------------------------------------------------------------------
  
  def getSession (self):
    # set up login handler
    self.connections.setLoginHandler (testLoginHandler ())

    print "Creating session object ..."
    return geasSession.geasSession (self.connections)
    
  # ---------------------------------------------------------------------------
  # Main program
  # ---------------------------------------------------------------------------
  
  def run (self):
    # created extra method to get the session, which can be overwritten
    # by testRpcApp
    session = self.getSession()
    
    print "Logging into the session ..."
    session.login ("hacker", "secret")

    print "Creating list object ..."
    list = session.createList ("person")

    print "Setting up list object ..."
    list.setPrefetch (["name", "street", "city"])
    list.setConditions ([])
    list.setSort (["zip"])

    print "Populating list ..."
    list.populate ()

    print "Retrieving first instance ..."
    instance = list.firstInstance ()

    print "These are the values of the first instance:"
    print "  Name  :", instance.get ("name")
    print "  Street:", instance.get ("street")
    print "  City  :", instance.get ("city")

    print "Check if person live in a rich/poor area:"
    print "  calling 'checkAreaRichness' results in: ", 
    instance.call('checkAreaRichness','') # demographic data

    print "Thank you for playing!"

# =============================================================================
# Login Handler
# =============================================================================

class testLoginHandler (GLoginHandler.LoginHandler):
  def getLogin (self, *arguments):
    # for this test program, try to log into the database as the current user
    username = os.environ ["LOGNAME"]
    return {"_username": username, "_password": "(none)"}

# =============================================================================
# Let it rock
# =============================================================================


def run():
  testApp().run()

if __name__ == "__main__":
  run()
